import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.media.*;
/**
 * Draw a sound waveform graph
 * @author Yang Yili
 */
public class GraphPanel extends JPanel
{
    /**
     * Create a new GraphPanel
     */
    public GraphPanel()
    {
		size = new Dimension(0,0);
		Dimension minimumSize = new Dimension(200, 100);
		setMinimumSize(minimumSize);
    }
    
    public void setBegin(int b){
	    begin = b;
    }
    
    public void setEnd(int e){
	    end = e;
    }

    /**
     * Get a reference of object FastFourierTransform
     * @return fastFourierTransform a reference of object FastFourierTransform
     */
    public FastFourierTransform getFFT()
    {
		return fastFourierTransform;
    }
	

    public double getDuration()
    {
		return (time);
    }
    /**
     * Get SamplingRate
     * @return samplingRate SamplingRate
     */
    public double getSamplingRate()
    {
		return(samplingRate);
    }
	
    /**
     * Get FrameLength
     * @return frameLength FrameLength
     */
    public long getFrameLength()
    {
		return(frameLength);
    }

    public double getFrameRate()
    {
		return(frameRate);
    }	

    /**
     * Get NumberOfSamples
     * @return numberOfSamples NumberOfSamples
     */
    public int getNumberOfSamples()
    {
		return(numberOfSamples);
    }
    
    /**
     * Clean the screen
     * @param fft a reference of object FastFourierTransform
     */
    public void draw(FastFourierTransform fft, double sampleRate,
		     		 double duration, int n, Reader r)
    {
		fastFourierTransform = fft;
		samplingRate = sampleRate;
		time = duration;
		numberOfFiles = n;
		reader = r;
		repaint();
    } 
    
    public void draw(FastFourierTransform fft, double sampleRate,
		     		 double duration, int n, Reader r1, Reader r2)
    {
		fastFourierTransform = fft;
		samplingRate = sampleRate;
		time = duration;
		numberOfFiles = n;
		reader1 = r1;
		reader2 = r2;
		repaint();
    } 

    /**
     * Draw graph
     * @param g Graphics
     */
    public void paintComponent(Graphics g)
    {
		rectangle = getBounds();
		size.width = (int)width;
		size.height = (int)height;
		setPreferredSize(size);
		revalidate();	
		super.paintComponent(g);
		if (fastFourierTransform == null)
		{
		}
	else
	{
	    //draw axis Y
	    g.drawLine(rectangle.x + 50, rectangle.y + 50,
		           rectangle.x + 50, rectangle.y + rectangle.height - 50);
		
	    //arrow
	    g.drawLine(rectangle.x + 50, rectangle.y + 50,
		           rectangle.x + 53, rectangle.y + 55);
	    g.drawLine(rectangle.x + 50, rectangle.y + 50,
		           rectangle.x + 47, rectangle.y + 55);
	    Font f1 = new Font("dialog", Font.BOLD, 14);
	    g.setFont(f1);
	    g.drawString("Hz", rectangle.x + 40,rectangle.y + 40);

	    //draw axis X
	    g.drawLine(rectangle.x + 50, rectangle.y + rectangle.height - 50,
		           rectangle.x + rectangle.width - 50, 
		           rectangle.y + rectangle.height - 50);
	    //arrow
	    g.drawLine(rectangle.x + rectangle.width - 50, 
		           rectangle.y + rectangle.height - 50,
		           rectangle.x + rectangle.width - 55, 
		           rectangle.y + rectangle.height - 53);
	    g.drawLine(rectangle.x + rectangle.width - 50, 
		           rectangle.y + rectangle.height - 50,
		           rectangle.x + rectangle.width - 55, 
		           rectangle.y + rectangle.height - 47);
	    g.drawString("S", rectangle.x + rectangle.width - 45, 
			              rectangle.y + rectangle.height - 45);

	    //axis Y
	    int totalDistance = rectangle.height - 120;
	    if (totalDistance < 0)
			totalDistance = 0;
	    int numberOfPoints = totalDistance / 40;
	    if (numberOfPoints == 0)
			numberOfPoints = 1;
	    int minFre = (int) samplingRate / (2 * numberOfPoints);
	    minFre = (minFre / 500 + 1) * 500;
	    
	    int n = (int)samplingRate / (2 * minFre);
	    if ( n == 0)
	    {
		minFre = (int) samplingRate / (2 * numberOfPoints);
		if (minFre == 0)
		    minFre = 1;
		n = (int)samplingRate / (2 * minFre);
		for (int i = 1; i <= n; i ++)
		{
		    double aux1 = (double)(rectangle.height - 120) 
			* 2 / samplingRate;
		    aux1 *= minFre * i;
		    int y = (int) aux1;
		    y = rectangle.y + rectangle.height - 50 - y;
		    g.drawLine(rectangle.x + 45,
			       y,
			       rectangle.x + 50,
			       y);
		    Font f = new Font("dialog", Font.PLAIN, 14);
		    g.setFont(f);
		    g.drawString("" + minFre * i,  
				 rectangle.x + 2,
				 y);
		}
		n = 0;
	    }
	    for (int i = 1; i <= n; i ++)
	    {
		double aux1 = (double)(rectangle.height - 120) * 
		    2 / samplingRate;
		aux1 *= minFre * i;
		int y = (int) aux1;
		y = rectangle.y + rectangle.height - 50 - y;
		g.drawLine(rectangle.x + 50, y,
			   rectangle.x + 45, y);
		Font f = new Font("dialog", Font.PLAIN, 14);
		g.setFont(f);
		g.drawString("" + minFre * i,  rectangle.x + 2, y);
	    }	

	    //axis X
	    totalDistance = rectangle.width - 120;
	    if (totalDistance < 0)
			totalDistance = 0;
	    numberOfPoints = totalDistance / 40;
	    if (numberOfPoints == 0)
			numberOfPoints = 1;
	    double minTime = time / (double) numberOfPoints;
	    minTime = (double) (((int)(minTime / 0.5) + 1) * 0.5);
	    n = (int)(time / minTime);
	    if ( n == 0)
	    {
		minTime = (double) (((int) time / numberOfPoints) * 10) / 10.0;
		if (minTime == 0.0)
		    minTime = 0.1;
		n = (int)(time / minTime);
		if (n == 0)
		    n = (int)(time / 0.1);
		for (int i = 1; i <= n; i ++)
		{
		    double aux1 = (double)(rectangle.width - 120) / time;
		    aux1 *= minTime * i;
		    int x = (int) aux1;
		    x = rectangle.x + 50 + x;
		    g.drawLine(x, rectangle.y + rectangle.height - 45,
			           x, rectangle.y + rectangle.height - 50);
		    Font f = new Font("dialog", Font.PLAIN, 14);
		    g.setFont(f);
		    double t = (double) (minTime * i);
		    int aux = (int)(t * 10.0);
		    t = (double) ((double)aux / 10.0);
		    g.drawString("" + (double) t, x,
				         rectangle.y + rectangle.height - 35);
		}
		n = 0;
	    }
	    for (int i = 1; i <= n; i ++)
	    {
		double aux1 = (double)(rectangle.width - 120) / time;
		aux1 *= minTime * i;
		int x = (int) aux1;
		x = rectangle.x + 50 + x;
		g.drawLine(x,
			   rectangle.y + rectangle.height - 45,
			   x,
			   rectangle.y + rectangle.height - 50);
		Font f = new Font("dialog", Font.PLAIN, 14);
		g.setFont(f);
		double t = (double) (minTime * i);
		int aux = (int)(t * 10.0);
		t = (double) ((double)aux / 10.0);
		g.drawString("" + t, x,
			     rectangle.y + rectangle.height - 35);
	    }
	    if (numberOfFiles == 1){
		    //while(spectrum.size() == 0)
	       	spectrum = reader.getSpectrum();
    	}
	    if (numberOfFiles == 2){
		    spectrum1 = reader1.getSpectrum();
		    spectrum2 = reader2.getSpectrum();
		    double[] tempSpectrum1;
		    double[] tempSpectrum2;
		    double[] difSpectrum = new double[512/2];
		    if((spectrum1.size()==spectrum2.size())&&(spectrum1.size()>0)){
		    	for(int i = 0; i < spectrum1.size(); i++){
			    	tempSpectrum1 = (double[])spectrum1.get(i);
			    	tempSpectrum2 = (double[])spectrum2.get(i);
			    	for(int j = 0; j < (512/2); j++)
			    		difSpectrum[j] = tempSpectrum2[j] - tempSpectrum1[j];
			    	spectrum.add(difSpectrum);
		    	}
	    	}
    	}
    	
	    double[] tempSpectrum;
	    double min =   Double.MAX_VALUE;
	    double max =  -Double.MAX_VALUE;
	    for (int x = 0; x < spectrum.size(); x++)
	    {
			tempSpectrum = (double[])spectrum.get(x);
			for(int y = 0; y < (512/2); y++){
			    if (max < tempSpectrum[y])
					max = tempSpectrum[y];
		    	if (min > tempSpectrum[y]) 
					min = tempSpectrum[y];
			}
	    }
	    Color maxColor = new Color(0.0f, 0.0f, 0.0f);
	    for (int x = 0; x < spectrum.size(); x++)
	    {
			double interval = (double) (rectangle.width - 120) / (double)spectrum.size();
			tempSpectrum = (double[])spectrum.get(x);
			for (int j = 0; j < 512 / 2; j ++)
			{
				double aux = (rectangle.height - 120) * 2 * j  / 512.0;
		    	int y = (int) aux;
		    	double percent = tempSpectrum[j] / max;
		    	int bright = 14 - (int)(percent * 14);
		    	Color color = maxColor;
		    	for (int k = 0; k < bright; k ++)
					color = color.brighter();
		    	g.setColor(color);
		    	g.drawLine(rectangle.x + 50 + (int)((double)x * interval), 
			    		   rectangle.y + rectangle.height - 50 - y, 
			      		   rectangle.x + 50 + (int)((double)(x + 1) * interval), 
			               rectangle.y + rectangle.height - 50 - y);
			}
	    }
	}
	} /* method paintComponent */
    
    private Vector spectrum = new Vector();
    private Vector spectrum1 = new Vector();
    private Vector spectrum2 = new Vector();
    private RecordPlay recordPlay;
    private Reader reader;
    private Reader reader1;
    private Reader reader2;
    private FastFourierTransform fastFourierTransform = null;
    private Dimension size;
    private long frameLength = (long)0.0;
    private double time = 0.0;
    private double frameRate = 0.0;
    private double samplingRate = 0.0;
    private double width = 0.0;
    private double height = 0.0;
    private int numberOfSamples = 0;
    private int numberOfFiles = 0;
    private int begin = 0;
    private int end = 0;
    private double frequency;
    private Rectangle rectangle = new Rectangle();
}/* class GraphPanel */










