/**
 * WindowFunction.java
 * Implements Static Methods for Window Functions for
 * Fast Fourier Transformation (FFT)
 */

public class WindowFunction  {

//-----------------------------------------------------------------------------
    public static double[] hammingWindow(int len) {
	double[] hamming = new double[len];
	for (int n=0;n<len;n++){
	    hamming[n]=0.5f-0.5f*(double)Math.cos(2.0*Math.PI*n/len);
	}
	return hamming;
    }
//-----------------------------------------------------------------------------
    public static double[] rectangleWindow(int len) {
	double[] rectangle = new double[len];
	for (int n=0;n<len;n++){
	    rectangle[n] = 1.0f;
	}
	return rectangle;
    }
//-----------------------------------------------------------------------------
    public static double[] blackmanWindow(int len) {
	double[] blackman = new double[len];
	for (int n=0;n<len;n++){
	    blackman[n] = 0.42f-(double)(0.5*Math.cos(2.0*Math.PI*n/len)+0.08*Math.cos(2.0*Math.PI*n/len));
	}
	return blackman;
    }
//-----------------------------------------------------------------------------
    public static double[] henningWindow(int len) {
	double[] henning = new double[len];
	for (int n=0;n<len;n++){
	    henning[n] = 0.54f-(double)(0.46*Math.cos(2.0*Math.PI*n/len));
	}
	return henning;
    }
//-----------------------------------------------------------------------------
    public static double[] triangleWindow(int len) {
	double[] triangle = new double[len];
	for (int n=0;n<len;n++){
	    if (n<len/2){
		triangle[n] = (double)n/(double)len * 2.0f;
	    }
	    else {
		triangle[n] = 2.0f - (double)(n) / (double)len * 2.0f;
	    }
	}
	return triangle;
    }
//-----------------------------------------------------------------------------
    public static double[] welchWindow(int len) {
	double[] welch = new double[len];
	for (int n=0;n<len;n++){
	    welch[n] = 1.0f - (double)Math.pow(((double)n - (double)len / 2.0) / ((double)len / 2.0),2.0);
	}
	return welch;
    }
//-----------------------------------------------------------------------------
    public static double[] gaussWindow(int len) {
	double[] gauss = new double[len];
	// Parameter der Gausskurve
	double rho = 0.3333;       
	for (int n=0;n<len;n++){
	    gauss[n] = (double)Math.exp(-0.5*Math.pow((n-len/2.0)/(rho*len/2.0),2));
	}
	return gauss;
    }
//-----------------------------------------------------------------------------
} /* WindowFunction */
